package Environment;

import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

public class Tile
{
	private final int XPOS;
	private final int YPOS;

	private TileType _type;
	private Map<Direction, Tile> _tiles;
	private boolean _visited;
	// Die Suchtiefe für die Breitensuche in der sich das Tile befindet
	private int _bsDepth;

	public Tile(int xPos, int yPos, TileType type)
	{
		XPOS = xPos;
		YPOS = yPos;
		_type = type;
		_tiles = new HashMap<Direction, Tile>();
		_bsDepth = -1;
		_visited = false;
	}

	public void setVisited(boolean b)
	{
		_visited = b;
	}

	public boolean visited()
	{
		return _visited;
	}

	public void setDepth(int t)
	{
		_bsDepth = t;
	}

	public int getDepth()
	{
		return _bsDepth;
	}

	public void addTile(Tile tile, Direction direction)
	{
		_tiles.put(direction, tile);
	}

	public Map<Direction, Tile> getNeighbours()
	{
		return _tiles;
	}

	/**
	 * Gibt die Anzahl der noch nicht in der Tiefensuche besuchten Nachbarknoten
	 * zurück
	 * 
	 * @return Anzahl noch nicht besuchter Nachbarn
	 */
	public int getUnvisitedNeighboursCount()
	{
		Tile t;
		int count = 0;

		for (Entry<Direction, Tile> entry : _tiles.entrySet())
		{
			t = (Tile) entry.getValue();
			if (!t.visited())
			{
				++count;
			}
		}
		return count;
	}

	public int getXPos()
	{
		return XPOS;
	}

	public int getYPos()
	{
		return YPOS;
	}

	public void setType(TileType t)
	{
		_type = t;
	}

	public TileType getType()
	{
		return _type;
	}

	@Override
	public String toString()
	{
		return "[" + getXPos() + ", " + getYPos() + ", " + getType() + "]";
	}
}
