package search;

import java.util.LinkedList;
import java.util.Map.Entry;
import java.util.Queue;

import Environment.Direction;
import Environment.Tile;
import Environment.TileType;

/**
 * Diese Klasse implemtiert die Breitesuche des Roboters
 * 
 * @author marco
 * 
 */
public class BreadthFirst extends Search
{
	private Queue<Tile> _queue;

	public BreadthFirst()
	{
		super("Breadth First Search");
		_queue = new LinkedList<Tile>();
	}

	@Override
	public void search(Tile startTile)
	{
		Tile parent;
		Tile child;

		startTile.setDepth(0);
		startTile.setVisited(true);
		_queue.add(startTile);

		// Solange die Queue nicht leer ist, wird jeweils der erste Eintrag der
		// Queue expandiert und dessen Kinderknoten an die Queue gehängt
		while (!_queue.isEmpty())
		{
			parent = _queue.remove();

			if (parent.getType().equals(TileType.GOAL))
			{
				// Knoten ist das Ziel, brich die Suche ab und baue den Pfad
				// zusammen
				buildPath(parent);
				_queue.clear();
			} else
			{
				// Knoten ist nicht das Ziel, expandiere ihn
				for (Entry<Direction, Tile> e : parent.getNeighbours()
						.entrySet())
				{
					child = (Tile) e.getValue();

					// Knoten noch nicht besucht, fuege Ihn in die Schlange
					if (!child.visited())
					{
						child.setVisited(true);
						child.setDepth(parent.getDepth() + 1);
						_queue.add(child);
					}
				}
			}
		}
	}

	public void buildPath(Tile t)
	{
		getFoundPath().add(0, t);

		while (t.getDepth() != 0)
		{
			for (Entry<Direction, Tile> e : t.getNeighbours().entrySet())
			{
				Tile entry = (Tile) e.getValue();
				if (entry.getDepth() == t.getDepth() - 1)
				{
					if (!(entry.getType().equals(TileType.START) || entry
							.getType().equals(TileType.GOAL)))
					{
						entry.setType(TileType.PATH);
					}
					getFoundPath().add(0, entry);
					t = entry;
				}
			}
		}
	}
}
