package search;

import java.util.Map.Entry;
import java.util.Stack;

import Environment.Direction;
import Environment.Tile;
import Environment.TileType;

/**
 * Diese Klasse implementiert die Tiefensuche des Roboters
 * 
 * @author marco
 * 
 */
public class DepthFirst extends Search
{
	private Stack<Tile> _stack;

	public DepthFirst()
	{
		super("Depth First Search");
		_stack = new Stack<Tile>();
	}

	@Override
	public void search(Tile startTile)
	{
		startTile.setVisited(true);
		_stack.push(startTile);

		while (!_stack.isEmpty())
		{
			// Das oberste Tile vom Stack nehmen und prüfen, ob es das Ziel ist.
			// Wenn, ja, Suche erfolgreich beenden, wenn nein, Tile expandieren
			Tile tile = _stack.peek();
			if (tile.getType().equals(TileType.GOAL))
			{
				buildPath();
				return;
			}
			// Wenn es noch mindestens einen unbesuchten Nachbarknoten gibt,
			// lege ihn für die weitere Untersuchung auf den Stack. Falls nicht,
			// nehme das aktuelle Teil vom Stack, da es komplett untersucht
			// wurde
			if (tile.getUnvisitedNeighboursCount() > 0)
			{
				for (Entry<Direction, Tile> entry : tile.getNeighbours()
						.entrySet())
				{
					Tile neighbour = (Tile) entry.getValue();
					if (!neighbour.visited())
					{
						neighbour.setVisited(true);
						_stack.push(neighbour);
						break;
					}
				}
			} else
			{
				_stack.pop();
			}
		}
	}

	public void buildPath()
	{
		while (!_stack.empty())
		{
			if (_stack.peek().getType().equals(TileType.GROUND))
			{
				_stack.peek().setType(TileType.PATH);
			}
			getFoundPath().add(0, _stack.pop());
		}
	}
}
